
// DentalCRSDKDemoDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DentalCRSDKDemo.h"
#include "DentalCRSDKDemoDlg.h"
#include "afxdialogex.h"
#include "SDKString.h"
#include "ImageListDlg.h"
#include "ScannerListDlg.h"

#define  WM_SCANNER_NOTIFY       (WM_APP + 0x301)
#define  WM_SCANNER_SCANNERLIST  (WM_APP + 0x302)
#define  WM_SCANNER_STATUS       (WM_APP + 0x303)

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ID_TIMER_SCAN   1

static TCallBackSet	callbackSet;
static int acqStep, scanStepCnt;

static const COLORREF scannerStsTxtClr[] =
{
   0x000000,  // None
   0x000000,  // Disconnected
   0x000000,  // Waiting
   0x000000,  // Synchronizing
   0x25B006,  // Connected
   0x000000,  // Sleeping
   0x25B006,  // Transferring
   0x25B006,  // Scanning
   0x25B006   // Erasing
};

static const COLORREF RFIDStsTxtClr[] =
{
   0x000000,  // RFID None
   0x000000,  // RFID Disconnected
   0x25B006   // RFID Connected
};


static CDentalCRSDKDemoDlg* MainForm()
{
   CWnd* wnd = AfxGetApp()->GetMainWnd();
   if (wnd && IsWindow(wnd->GetSafeHwnd()))
      return static_cast<CDentalCRSDKDemoDlg*>(wnd);
   return nullptr;
}

void cdecl OnRFIDStatus(int status)
{
   // notification for FireID's connection
	MainForm()->UpdateRFIDStatus(status);
}

void cdecl OnRFIDNotify(int notify)
{
   // notification for IP on FireID, on IP tray, or scanned
	MainForm()->UpdateRFIDNotify(notify);
}

void cdecl OnScannerStatus(int status)
{
	MainForm()->UpdateScannerStatus(status);
}

void cdecl OnScannerList(int count)
{
   MainForm()->PostMessage(WM_SCANNER_SCANNERLIST, count);
}

void cdecl OnImageList(int /*count*/)
{
}

void cdecl OnScannerNotify(int notify)
{
   LPARAM lParam = 0;

	switch(notify) {
	// transferring progress
   case esnProgress:
      lParam = GetProgress();
		break;
	case esnPreprocessed:
      lParam = (LPARAM)MainForm()->SetImageBuf(GetImageWidth(), GetImageHeight(), 
         GetImageBuffer(), GetPatientID(ernFromImage));
		break;
   // IP size or scan resolution changed
	case esnChangeFormat:
   {
      EIPSize ipSize = GetIPSize();
      lParam = MAKELPARAM((WORD)ipSize, (WORD)GetImageResolution());

      // set patient ID to the scanner when a IP on the tray
      if (eisNone != ipSize)
         MainForm()->SetPatientInfo();
   }
		break;
	case esnScanStarted:
      MainForm()->SetInProgress(INPROGRESS_SCANIMAGE);
      break;
	case esnScannerError:
      lParam = GetScannerError();
      break;
	case esnCalPassed:
	case esnCalFailed:
		break;
	}

   MainForm()->PostMessage(WM_SCANNER_NOTIFY, notify, lParam);
}

// copy scanned image information and image data
ImageInfo* CDentalCRSDKDemoDlg::SetImageBuf(int width, int height, PIXEL_FORMAT* src, 
   const char* patId)
{
   ASSERT(m_pImgInfo == nullptr);
   m_pImgInfo = new ImageInfo(width, height);
   memcpy(m_pImgInfo->data, src, width * height * sizeof(PIXEL_FORMAT));

   if (patId)
      m_pImgInfo->patientId = patId;

   return m_pImgInfo;
}


// CDentalCRSDKDemoDlg dialog


CDentalCRSDKDemoDlg::CDentalCRSDKDemoDlg(CWnd* pParent /*=NULL*/)
   : CDialogEx(IDD_DENTALCRSDKDEMO_DIALOG, pParent)
   , m_ScannerState(estNone)
   , m_pImageListDlg(nullptr)
   , m_pScannerListDlg(nullptr)
   , m_pImgInfo(nullptr)
   , m_inProgress(INPROGRESS_NONE)
   , m_RFIDStatus(ersNone)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CDentalCRSDKDemoDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialogEx::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_PROGRESS_SCANNER, m_progress);
}

BEGIN_MESSAGE_MAP(CDentalCRSDKDemoDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BTN_CLOSESCANNER, &CDentalCRSDKDemoDlg::OnBnClickedBtnCloseScanner)
	ON_BN_CLICKED(IDC_BTN_OPENSCANNER, &CDentalCRSDKDemoDlg::OnBnClickedBtnOpenScanner)
	ON_BN_CLICKED(IDC_BTN_CALIBRATION, &CDentalCRSDKDemoDlg::OnBnClickedBtnCalibration)
	ON_BN_CLICKED(IDC_BTN_SCANNERCONTROL, &CDentalCRSDKDemoDlg::OnBnClickedBtnScannercontrol)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BTN_IMAGELIST, &CDentalCRSDKDemoDlg::OnBnClickedBtnImageList)
	ON_BN_CLICKED(IDC_BTN_SLEEP, &CDentalCRSDKDemoDlg::OnBnClickedBtnSleep)
	ON_BN_CLICKED(IDC_BTN_SCANNERLIST, &CDentalCRSDKDemoDlg::OnBnClickedBtnScannerlist)
   ON_MESSAGE(WM_SCANNER_NOTIFY, OnScannerNotifyMsg)
   ON_MESSAGE(WM_SCANNER_SCANNERLIST, OnScannerScannerListMsg)
   ON_MESSAGE(WM_SCANNER_STATUS, OnScannerStatusMsg)
   ON_BN_CLICKED(IDC_BTN_SETPATIENT, &CDentalCRSDKDemoDlg::OnBnClickedBtnSetpatient)
   ON_WM_TIMER()
   ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CDentalCRSDKDemoDlg message handlers

BOOL CDentalCRSDKDemoDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

   EnableButtons(FALSE);

   // set callback functions
	callbackSet.rfidNotifyEvent = (TOnRFIDNotifyEvent)OnRFIDNotify;
	callbackSet.rfidStatusEvent = (TOnRFIDStatusEvent)OnRFIDStatus;
	callbackSet.scannerNotifyEvent = (TOnScannerNotifyEvent)OnScannerNotify;
	callbackSet.scannerStatusEvent = (TOnScannerStatusEvent)OnScannerStatus;
   callbackSet.scannerListEvent = (TOnScannerListEvent)OnScannerList;
   callbackSet.imageListEvent = (TOnImageListEvent)OnImageList;

   if (OpenScannerSDK(m_hWnd, &callbackSet, _T("."))) {
      UpdateScannerStatus(estNone);

      if (!m_pScannerListDlg) {
         m_pScannerListDlg = new CScannerListDlg(this);
         m_pScannerListDlg->Create(IDD_SCANNER_LIST, this);
         m_pScannerListDlg->InitScannerList();
      }
      if (!m_pImageListDlg) {
         m_pImageListDlg = new  CImageListDlg(this);
         m_pImageListDlg->Create(IDD_IMAGELIST_DIALOG, this);
         m_pImageListDlg->InitList();
      }

      m_progress.SetRange(0, 100);
      m_progress.SetPos(0);

      GetDlgItem(IDC_EDITPATIENT_ID)->SetWindowText(_T("Patient1"));
   }
   else {
      MessageBox(_T("Failed to open scanner SDK."), _T("Open Scanner SDK"), MB_OK);
      GetDlgItem(IDC_BTN_OPENSCANNER)->EnableWindow(FALSE);
   }

   return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CDentalCRSDKDemoDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CDentalCRSDKDemoDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnCloseScanner()
{
	CloseScanner();

   UpdateScannerStatus(estNone);
   EnableButtons(FALSE);

   if (m_pScannerListDlg)
      m_pScannerListDlg->OnBnClickedUpdateList();
   if (m_pImageListDlg)
      m_pImageListDlg->ShowWindow(SW_HIDE);
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnOpenScanner()
{
	if (OpenScanner())
      EnableButtons(TRUE);
}

void CDentalCRSDKDemoDlg::EnableButtons(BOOL bConnected)
{
   GetDlgItem(IDC_BTN_OPENSCANNER)->EnableWindow(!bConnected);
   GetDlgItem(IDC_BTN_CLOSESCANNER)->EnableWindow(bConnected);
   GetDlgItem(IDC_BTN_CALIBRATION)->EnableWindow(bConnected);
   GetDlgItem(IDC_BTN_SCANNERCONTROL)->EnableWindow(bConnected);
   GetDlgItem(IDC_BTN_SCANNERLIST)->EnableWindow(bConnected);
   GetDlgItem(IDC_BTN_SLEEP)->EnableWindow(bConnected);
   GetDlgItem(IDC_BTN_IMAGELIST)->EnableWindow(bConnected);
   GetDlgItem(IDC_EDITPATIENT_ID)->EnableWindow(bConnected);
   GetDlgItem(IDC_BTN_SETPATIENT)->EnableWindow(bConnected);
}

bool CDentalCRSDKDemoDlg::CheckConnected(CWnd* pWnd)
{
   if (IsConnectedState())
      return true;

   if (pWnd == nullptr)
      pWnd = this;
   // if not connected state
   pWnd->MessageBox(_T("Scanner is not \'Connected\' state."), _T("State Check"), MB_OK | MB_ICONWARNING);
   return false;
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnCalibration()
{
   if (!CheckConnected()) return;

	if (!ShowCalibrationDialog(true))
		MessageBox(_T("Calibration is not done"), _T("Check Calibration"),
         MB_OK | MB_ICONWARNING);
}

void CDentalCRSDKDemoDlg::UpdateRFIDNotify(int notify)
{
   CString buf;

	TUID uid = GetUID((ERFIDNotify)notify);

   switch (notify) {
	case ernFromFireID:
      buf.Format(_T("%I64X(FireID)"), uid);
      break;
   case ernFromScanner:
      buf.Format(_T("%I64X(Scanner)"), uid);

      if (0 < uid)
         ResetTransferProgressUI();
      break;
   case ernFromImage:
      buf.Format(_T("%I64X(Image)"), uid);
      break;
   case ernFromOtherSDK:
      buf.Format(_T("%I64X(Other)"), uid);
      break;
	case ersNone:
      buf.Format(_T("%I64X(Other)"), uid);
      break;
	}

	GetDlgItem(IDC_STATIC_TAGGED)->SetWindowText(buf);
}

void CDentalCRSDKDemoDlg::UpdateRFIDStatus(int status)
{
   m_RFIDStatus = (ERFIDStatus)status;
	GetDlgItem(IDC_STATUS_RFID)->SetWindowText(RFIDStatusMsg[status]);

   if (ersDisconnected == m_RFIDStatus)
      GetDlgItem(IDC_STATIC_TAGGED)->SetWindowText(_T(""));
}

void CDentalCRSDKDemoDlg::UpdateScannerStatus(int status)
{
   m_ScannerState = (EScannerStatus)status;
	GetDlgItem(IDC_STATUS_SCANNER)->SetWindowText(scannerStatusMsg[status]);

	switch(status) {
	case estNone:
		break;
	case estDisconnected:
		break;
	case estWaiting:
		GetDlgItem(IDC_STATUS_CONNECTION)->SetWindowText(ConnectionList[GetScannerConnectionType()]);
		break;
   case estSynchronizing:
      break;
	case estConnected:
      GetDlgItem(IDC_STATUS_CONNECTION)->SetWindowText(ConnectionList[GetScannerConnectionType()]);
		break;
   case estSleeping:
      break;
	case estTransferring:
      break;
   case estScanning:
      SetInProgress(INPROGRESS_SCANIMAGE);
      break;
   case estErasing:
      break;
	}

   PostMessage(WM_SCANNER_STATUS, status);
}

void CDentalCRSDKDemoDlg::ResetScanInfoUI()
{
   m_progress.SetPos(0);
   GetDlgItem(IDC_STATIC_IPSIZE)->SetWindowText(strIPSizeList[0]);
   GetDlgItem(IDC_STATIC_TAGGED)->SetWindowText(_T(""));
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnScannercontrol()
{
   if (!CheckConnected()) return;

	ShowScannerControlDialog(TRUE);
}

void CDentalCRSDKDemoDlg::OnClose()
{
	if (m_pImageListDlg) {
      m_pImageListDlg->DestroyWindow();
		delete m_pImageListDlg;
		m_pImageListDlg = NULL;
	}
	if (m_pScannerListDlg) {
      m_pScannerListDlg->DestroyWindow();
		delete m_pScannerListDlg;
		m_pScannerListDlg = NULL;
	}

	OnBnClickedBtnCloseScanner();
   CloseScannerSDK();

   CDialogEx::OnClose();
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnImageList()
{
   if (!CheckConnected()) return;

   if (m_pImageListDlg) {
      m_pImageListDlg->OnBnClickedUpdateList();
      m_pImageListDlg->ShowWindow(SW_NORMAL);
   }
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnSleep()
{
   if (GetScannerStatus() != estSleeping)
      if (!CheckConnected()) return;

   if (IsConnectedState())
      SleepScanner(TRUE);
   else if (GetScannerStatus() == estSleeping)
      SleepScanner(FALSE);
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnScannerlist()
{
   if (m_pScannerListDlg) {
      m_pScannerListDlg->OnBnClickedUpdateList();
      m_pScannerListDlg->ShowWindow(SW_NORMAL);
   }
}

LRESULT CDentalCRSDKDemoDlg::OnScannerNotifyMsg(WPARAM wParam, LPARAM lParam)
{
   switch (wParam) {
   case esnProgress:
      if (0 < scanStepCnt) {
         KillTimer(ID_TIMER_SCAN);
         scanStepCnt = 0;
      }

      if (IsScanOrGetImageInProgress()) {
         // some tweaks like (pos + 1) are included for CProgressCtrl object
         const int upper = 100;
         int pos = ((int)lParam +10) / 10;
         pos = acqStep + (int)((100.0 - acqStep) * pos / 100.0);

         if (pos < upper) {
            m_progress.SetPos(pos + 1);
            m_progress.SetPos(pos);
         }
         else {
            m_progress.SetRange(0, upper + 1);
            m_progress.SetPos(upper + 1);
            m_progress.SetPos(upper);
            m_progress.SetRange(0, upper);
         }
      }
      break;
   case esnPreprocessed:
   {
      ImageInfo* pImg = reinterpret_cast<ImageInfo*>(lParam);

      if (pImg->patientId) {
         CString buf;
         buf.Format(_T("%s(Image)"), LPCTSTR(CA2W(pImg->patientId, CP_UTF8)));
         GetDlgItem(IDC_PATIENT_ID)->SetWindowText(buf);
      }

      FILE* fp = NULL;
      if (_tfopen_s(&fp, _T("image.raw"), _T("wb")) == 0) {
         fwrite(pImg->data, pImg->width * pImg->height * sizeof(PIXEL_FORMAT), 1, fp);
         fclose(fp);
      }

      m_inProgress = INPROGRESS_NONE;

      // reset scanned image buffer
      delete m_pImgInfo;
      m_pImgInfo = nullptr;
   }
      break;
   case esnChangeFormat:
   {
      EIPSize ipSize = (EIPSize)LOWORD(lParam);
      GetDlgItem(IDC_STATIC_IPSIZE)->SetWindowText(strIPSizeList[ipSize]);
      GetDlgItem(IDC_STATIC_RESOLUTION)->SetWindowText(
         HIWORD(lParam) == esrHD ? _T("HD") : _T("SD"));

      if (eisNone == ipSize)
         m_progress.SetPos(0);
   }
      break;
   case esnScanStarted:
      SetTimer(ID_TIMER_SCAN, 500, NULL);
      ResetTransferProgressUI();
      GetDlgItem(IDC_STATUS_SCANNER)->SetWindowText(_T("Scanning"));
      break;
   case esnScannerError:
      switch (lParam) {
      case eseDoorJam:
         MessageBox(_T("Door Jam Error"), _T("Scanner Error"), MB_OK | MB_ICONERROR);
         break;
      case eseDuplicateSDK:
         MessageBox(_T("multiple SW is running"), _T("Scanner Error"), MB_OK | MB_ICONERROR);
         break;
      }
   case esnCalPassed:
   case esnCalFailed:
      break;
   }

   return 0;
}

LRESULT CDentalCRSDKDemoDlg::OnScannerScannerListMsg(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
   m_pScannerListDlg->OnBnClickedUpdateList();

   return 0;
}

LRESULT CDentalCRSDKDemoDlg::OnScannerStatusMsg(WPARAM wParam, LPARAM /*lParam*/)
{
   switch (wParam) {
   case estNone:
      break;
   case estDisconnected:
      ResetScanInfoUI();
      break;
   case estWaiting:
      break;
   case estSynchronizing:
      break;
   case estConnected:
      break;
   case estSleeping:
      break;
   case estTransferring:
      break;
   case estScanning:
      break;
   case estErasing:
      break;
   }

   return 0;
}

// set a patient ID to the scanner
// UTF-8 encoding is used in this sample
void CDentalCRSDKDemoDlg::SetPatientInfo()
{
   CString buf;

   GetDlgItem(IDC_EDITPATIENT_ID)->GetWindowText(buf);
   CStringA patId = CW2A(buf, CP_UTF8);

   SetPatientID(patId);
}

void CDentalCRSDKDemoDlg::OnBnClickedBtnSetpatient()
{
   if (!CheckConnected()) return;

   // clear previous patient ID
   GetDlgItem(IDC_PATIENT_ID)->SetWindowText(_T(""));

   SetPatientInfo();
}

void CDentalCRSDKDemoDlg::DisplayErrorMessage(const CString& title)
{
   CString errMsg = CA2W(GetErrorMessage(GetErrorCode()));
   MessageBox(errMsg, title, MB_OK | MB_ICONWARNING);
}

bool CDentalCRSDKDemoDlg::IsNetReady() const
{
   return (GetScannerConnectionType() == ectNtoN &&
      (estDisconnected == m_ScannerState || estWaiting == m_ScannerState));
}

void CDentalCRSDKDemoDlg::ResetTransferProgressUI()
{
   acqStep = scanStepCnt = 0;
   m_progress.SetPos(0);
   // clear previous information
   GetDlgItem(IDC_PATIENT_ID)->SetWindowText(_T(""));
}

void CDentalCRSDKDemoDlg::OnTimer(UINT_PTR nIDEvent)
{
   switch (nIDEvent)
   {
   case ID_TIMER_SCAN:
      ++scanStepCnt;
      acqStep = scanStepCnt * (66.7 / (10 / 0.5));
      m_progress.SetPos(acqStep);
      break;
   }

   CDialogEx::OnTimer(nIDEvent);
}

HBRUSH CDentalCRSDKDemoDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
   HBRUSH hbr = CDialogEx::OnCtlColor(pDC, pWnd, nCtlColor);

   if (nCtlColor == CTLCOLOR_STATIC) {
      if (pWnd->m_hWnd == GetDlgItem(IDC_STATUS_SCANNER)->m_hWnd)
         pDC->SetTextColor(scannerStsTxtClr[m_ScannerState]);
      else if (pWnd->m_hWnd == GetDlgItem(IDC_STATUS_RFID)->m_hWnd)
         pDC->SetTextColor(RFIDStsTxtClr[m_RFIDStatus]);
   }

   return hbr;
}
